@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  *
  ***************************************************************************/
 
@@ -125,8 +127,8 @@ convert(char *d, size_t dlen, int dccsid,
     dccsid = ASCII_CCSID;
 
   if(sccsid == dccsid) {
-    lslen = slen >= 0? slen: strlen(s) + 1;
-    i = lslen < dlen? lslen: dlen;
+    lslen = slen >= 0 ? slen : strlen(s) + 1;
+    i = lslen < dlen ? lslen : dlen;
 
     if(s != d && i > 0)
       memcpy(d, s, i);
@@ -168,7 +170,7 @@ static char *dynconvert(int dccsid, const char *s, int slen, int sccsid)
 
   /* Like convert, but the destination is allocated and returned. */
 
-  dlen = (size_t) (slen < 0? strlen(s): slen) + 1;
+  dlen = (size_t) (slen < 0 ? strlen(s) : slen) + 1;
   dlen *= MAX_CONV_EXPANSION;           /* Allow some expansion. */
   d = malloc(dlen);
 
@@ -233,28 +235,50 @@ slist_convert(int dccsid, struct curl_slist *from, int sccsid)
 }
 
 
-char *curl_version_ccsid(unsigned int ccsid)
+static char *
+keyed_string(localkey_t key, const char *ascii, unsigned int ccsid)
 {
   int i;
-  char *aversion;
-  char *eversion;
+  char *ebcdic;
 
-  aversion = curl_version();
-
-  if(!aversion)
-    return aversion;
-
-  i = strlen(aversion) + 1;
-  i *= MAX_CONV_EXPANSION;
-
-  eversion = Curl_thread_buffer(LK_CURL_VERSION, i);
-  if(!eversion)
+  if(!ascii)
     return (char *) NULL;
 
-  if(convert(eversion, i, ccsid, aversion, -1, ASCII_CCSID) < 0)
+  i = MAX_CONV_EXPANSION * (strlen(ascii) + 1);
+
+  ebcdic = Curl_thread_buffer(key, i);
+  if(!ebcdic)
+    return ebcdic;
+
+  if(convert(ebcdic, i, ccsid, ascii, -1, ASCII_CCSID) < 0)
     return (char *) NULL;
 
-  return eversion;
+  return ebcdic;
+}
+
+
+const char *
+curl_to_ccsid(const char *s, unsigned int ccsid)
+{
+  if(s)
+    s = dynconvert(ccsid, s, -1, ASCII_CCSID);
+  return s;
+}
+
+
+const char *
+curl_from_ccsid(const char *s, unsigned int ccsid)
+{
+  if(s)
+    s = dynconvert(ASCII_CCSID, s, -1, ccsid);
+  return s;
+}
+
+
+char *
+curl_version_ccsid(unsigned int ccsid)
+{
+  return keyed_string(LK_CURL_VERSION, curl_version(), ccsid);
 }
 
 
@@ -266,11 +290,12 @@ curl_easy_escape_ccsid(CURL *handle, const char *string, int length,
   char *d;
 
   if(!string) {
+    /* !checksrc! disable ERRNOVAR 1 */
     errno = EINVAL;
     return (char *) NULL;
     }
 
-  s = dynconvert(ASCII_CCSID, string, length? length: -1, sccsid);
+  s = dynconvert(ASCII_CCSID, string, length ? length : -1, sccsid);
 
   if(!s)
     return (char *) NULL;
@@ -296,11 +321,12 @@ curl_easy_unescape_ccsid(CURL *handle, const char *string, int length,
   char *d;
 
   if(!string) {
+    /* !checksrc! disable ERRNOVAR 1 */
     errno = EINVAL;
     return (char *) NULL;
     }
 
-  s = dynconvert(ASCII_CCSID, string, length? length: -1, sccsid);
+  s = dynconvert(ASCII_CCSID, string, length ? length : -1, sccsid);
 
   if(!s)
     return (char *) NULL;
@@ -412,18 +438,20 @@ curl_version_info_ccsid(CURLversion stamp, unsigned int ccsid)
     offsetof(curl_version_info_data, capath),
     offsetof(curl_version_info_data, zstd_version),
     offsetof(curl_version_info_data, hyper_version),
-    offsetof(curl_version_info_data, gsasl_version)
+    offsetof(curl_version_info_data, gsasl_version),
+    offsetof(curl_version_info_data, feature_names),
+    offsetof(curl_version_info_data, rtmp_version)
   };
 
   /* The assertion below is possible, because although the second operand
      is an enum member, the first is a #define. In that case, the OS/400 C
      compiler seems to compare string values after substitution. */
 
-#if CURLVERSION_NOW != CURLVERSION_TENTH
+#if CURLVERSION_NOW != CURLVERSION_ELEVENTH
 #error curl_version_info_data structure has changed: upgrade this procedure.
 #endif
 
-  /* If caller has been compiled with a new version, error. */
+  /* If caller has been compiled with a newer version, error. */
 
   if(stamp > CURLVERSION_NOW)
     return (curl_version_info_data *) NULL;
@@ -470,7 +498,7 @@ curl_version_info_ccsid(CURLversion stamp, unsigned int ccsid)
   memcpy((char *) id, (char *) p, sizeof(*p));
 
   if(id->protocols) {
-    int i = nproto * sizeof(id->protocols[0]);
+    i = nproto * sizeof(id->protocols[0]);
 
     id->protocols = (const char * const *) cp;
     memcpy(cp, (char *) p->protocols, i);
@@ -485,7 +513,7 @@ curl_version_info_ccsid(CURLversion stamp, unsigned int ccsid)
 
   for(i = 0; i < sizeof(charfields) / sizeof(charfields[0]); i++) {
     cpp = (const char **) ((char *) p + charfields[i]);
-    if (*cpp && convert_version_info_string(cpp, &cp, &n, ccsid))
+    if(*cpp && convert_version_info_string(cpp, &cp, &n, ccsid))
       return (curl_version_info_data *) NULL;
   }
 
@@ -496,75 +524,28 @@ curl_version_info_ccsid(CURLversion stamp, unsigned int ccsid)
 const char *
 curl_easy_strerror_ccsid(CURLcode error, unsigned int ccsid)
 {
-  int i;
-  const char *s;
-  char *buf;
-
-  s = curl_easy_strerror(error);
-
-  if(!s)
-    return s;
-
-  i = MAX_CONV_EXPANSION * (strlen(s) + 1);
-
-  buf = Curl_thread_buffer(LK_EASY_STRERROR, i);
-  if(!buf)
-    return (const char *) NULL;
-
-  if(convert(buf, i, ccsid, s, -1, ASCII_CCSID) < 0)
-    return (const char *) NULL;
-
-  return (const char *) buf;
+  return keyed_string(LK_EASY_STRERROR, curl_easy_strerror(error), ccsid);
 }
 
 
 const char *
 curl_share_strerror_ccsid(CURLSHcode error, unsigned int ccsid)
 {
-  int i;
-  const char *s;
-  char *buf;
-
-  s = curl_share_strerror(error);
-
-  if(!s)
-    return s;
-
-  i = MAX_CONV_EXPANSION * (strlen(s) + 1);
-
-  buf = Curl_thread_buffer(LK_SHARE_STRERROR, i);
-  if(!buf)
-    return (const char *) NULL;
-
-  if(convert(buf, i, ccsid, s, -1, ASCII_CCSID) < 0)
-    return (const char *) NULL;
-
-  return (const char *) buf;
+  return keyed_string(LK_SHARE_STRERROR, curl_share_strerror(error), ccsid);
 }
 
 
 const char *
 curl_multi_strerror_ccsid(CURLMcode error, unsigned int ccsid)
 {
-  int i;
-  const char *s;
-  char *buf;
+  return keyed_string(LK_MULTI_STRERROR, curl_multi_strerror(error), ccsid);
+}
 
-  s = curl_multi_strerror(error);
 
-  if(!s)
-    return s;
-
-  i = MAX_CONV_EXPANSION * (strlen(s) + 1);
-
-  buf = Curl_thread_buffer(LK_MULTI_STRERROR, i);
-  if(!buf)
-    return (const char *) NULL;
-
-  if(convert(buf, i, ccsid, s, -1, ASCII_CCSID) < 0)
-    return (const char *) NULL;
-
-  return (const char *) buf;
+const char *
+curl_url_strerror_ccsid(CURLUcode error, unsigned int ccsid)
+{
+  return keyed_string(LK_URL_STRERROR, curl_url_strerror(error), ccsid);
 }
 
 
@@ -1067,7 +1048,7 @@ Curl_formget_callback_ccsid(void *arg, const char *buf, size_t len)
 
   ret = (*p->append)(p->arg, b, l);
   free(b);
-  return ret == l? len: -1;
+  return ret == l ? len : -1;
 }
 
 
@@ -1085,22 +1066,25 @@ curl_formget_ccsid(struct curl_httppost *form, void *arg,
 
 
 CURLcode
-curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
+curl_easy_setopt_ccsid(CURL *easy, CURLoption tag, ...)
 {
   CURLcode result;
   va_list arg;
-  struct Curl_easy *data;
   char *s;
-  char *cp;
+  char *cp = NULL;
   unsigned int ccsid;
   curl_off_t pfsize;
+  struct Curl_easy *data = easy;
 
-  data = (struct Curl_easy *) curl;
   va_start(arg, tag);
 
   switch(tag) {
 
+  /* BEGIN TRANSLATABLE STRING OPTIONS */
+  /* Keep option symbols in alphanumeric order and retain the BEGIN/END
+     armor comments. */
   case CURLOPT_ABSTRACT_UNIX_SOCKET:
+  case CURLOPT_ACCEPT_ENCODING:
   case CURLOPT_ALTSVC:
   case CURLOPT_AWS_SIGV4:
   case CURLOPT_CAINFO:
@@ -1112,16 +1096,17 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
   case CURLOPT_CRLFILE:
   case CURLOPT_CUSTOMREQUEST:
   case CURLOPT_DEFAULT_PROTOCOL:
-  case CURLOPT_DNS_SERVERS:
   case CURLOPT_DNS_INTERFACE:
   case CURLOPT_DNS_LOCAL_IP4:
   case CURLOPT_DNS_LOCAL_IP6:
+  case CURLOPT_DNS_SERVERS:
   case CURLOPT_DOH_URL:
+  case CURLOPT_ECH:
   case CURLOPT_EGDSOCKET:
-  case CURLOPT_ENCODING:
   case CURLOPT_FTPPORT:
   case CURLOPT_FTP_ACCOUNT:
   case CURLOPT_FTP_ALTERNATIVE_TO_USER:
+  case CURLOPT_HAPROXY_CLIENT_IP:
   case CURLOPT_HSTS:
   case CURLOPT_INTERFACE:
   case CURLOPT_ISSUERCERT:
@@ -1135,6 +1120,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
   case CURLOPT_PASSWORD:
   case CURLOPT_PINNEDPUBLICKEY:
   case CURLOPT_PRE_PROXY:
+  case CURLOPT_PROTOCOLS_STR:
   case CURLOPT_PROXY:
   case CURLOPT_PROXYPASSWORD:
   case CURLOPT_PROXYUSERNAME:
@@ -1142,6 +1128,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
   case CURLOPT_PROXY_CAINFO:
   case CURLOPT_PROXY_CAPATH:
   case CURLOPT_PROXY_CRLFILE:
+  case CURLOPT_PROXY_ISSUERCERT:
   case CURLOPT_PROXY_KEYPASSWD:
   case CURLOPT_PROXY_PINNEDPUBLICKEY:
   case CURLOPT_PROXY_SERVICE_NAME:
@@ -1156,6 +1143,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
   case CURLOPT_PROXY_TLSAUTH_USERNAME:
   case CURLOPT_RANDOM_FILE:
   case CURLOPT_RANGE:
+  case CURLOPT_REDIR_PROTOCOLS_STR:
   case CURLOPT_REFERER:
   case CURLOPT_REQUEST_TARGET:
   case CURLOPT_RTSP_SESSION_ID:
@@ -1165,6 +1153,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
   case CURLOPT_SERVICE_NAME:
   case CURLOPT_SOCKS5_GSSAPI_SERVICE:
   case CURLOPT_SSH_HOST_PUBLIC_KEY_MD5:
+  case CURLOPT_SSH_HOST_PUBLIC_KEY_SHA256:
   case CURLOPT_SSH_KNOWNHOSTS:
   case CURLOPT_SSH_PRIVATE_KEYFILE:
   case CURLOPT_SSH_PUBLIC_KEYFILE:
@@ -1175,6 +1164,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
   case CURLOPT_SSLKEYTYPE:
   case CURLOPT_SSL_CIPHER_LIST:
   case CURLOPT_SSL_EC_CURVES:
+  case CURLOPT_SSL_SIGNATURE_ALGORITHMS:
   case CURLOPT_TLS13_CIPHERS:
   case CURLOPT_TLSAUTH_PASSWORD:
   case CURLOPT_TLSAUTH_TYPE:
@@ -1185,6 +1175,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
   case CURLOPT_USERNAME:
   case CURLOPT_USERPWD:
   case CURLOPT_XOAUTH2_BEARER:
+  /* END TRANSLATABLE STRING OPTIONS */
     s = va_arg(arg, char *);
     ccsid = va_arg(arg, unsigned int);
 
@@ -1197,7 +1188,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
       }
     }
 
-    result = curl_easy_setopt(curl, tag, s);
+    result = curl_easy_setopt(easy, tag, s);
     free(s);
     break;
 
@@ -1211,7 +1202,7 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
     pfsize = data->set.postfieldsize;
 
     if(!s || !pfsize || ccsid == NOCONV_CCSID || ccsid == ASCII_CCSID) {
-      result = curl_easy_setopt(curl, CURLOPT_COPYPOSTFIELDS, s);
+      result = curl_easy_setopt(easy, CURLOPT_COPYPOSTFIELDS, s);
       break;
     }
 
@@ -1222,8 +1213,8 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
       if(!s) {
         result = CURLE_OUT_OF_MEMORY;
         break;
-        }
       }
+    }
     else {
       /* Data length specified. */
       size_t len;
@@ -1249,30 +1240,69 @@ curl_easy_setopt_ccsid(CURL *curl, CURLoption tag, ...)
       pfsize = convert(cp, pfsize, ASCII_CCSID, s, len, ccsid);
 
       if(pfsize < 0) {
-        free(cp);
         result = CURLE_OUT_OF_MEMORY;
         break;
       }
 
       data->set.postfieldsize = pfsize;         /* Replace data size. */
       s = cp;
+      cp = NULL;
     }
 
-    result = curl_easy_setopt(curl, CURLOPT_POSTFIELDS, s);
+    result = curl_easy_setopt(easy, CURLOPT_POSTFIELDS, s);
     data->set.str[STRING_COPYPOSTFIELDS] = s;   /* Give to library. */
     break;
 
-  case CURLOPT_ERRORBUFFER:                     /* This is an output buffer. */
   default:
-    result = Curl_vsetopt(curl, tag, arg);
+    if(tag / 10000 == CURLOPTTYPE_BLOB) {
+      struct curl_blob *bp = va_arg(arg, struct curl_blob *);
+      struct curl_blob blob;
+
+      ccsid = va_arg(arg, unsigned int);
+
+      if(bp && bp->data && bp->len &&
+         ccsid != NOCONV_CCSID && ccsid != ASCII_CCSID) {
+        pfsize = (curl_off_t) bp->len * MAX_CONV_EXPANSION;
+
+        if(pfsize > SIZE_MAX)
+          pfsize = SIZE_MAX;
+
+        cp = malloc(pfsize);
+
+        if(!cp) {
+          result = CURLE_OUT_OF_MEMORY;
+          break;
+        }
+
+        pfsize = convert(cp, pfsize, ASCII_CCSID, bp->data, bp->len, ccsid);
+
+        if(pfsize < 0) {
+          result = CURLE_OUT_OF_MEMORY;
+          break;
+        }
+
+        blob.data = cp;
+        blob.len = pfsize;
+        blob.flags = bp->flags | CURL_BLOB_COPY;
+        bp = &blob;
+      }
+      result = curl_easy_setopt(easy, tag, bp);
+      break;
+    }
+    FALLTHROUGH();
+  case CURLOPT_ERRORBUFFER:                     /* This is an output buffer. */
+    result = Curl_vsetopt(easy, tag, arg);
     break;
   }
 
   va_end(arg);
+  free(cp);
   return result;
 }
 
 
+/* ILE/RPG helper functions. */
+
 char *
 curl_form_long_value(long value)
 {
@@ -1282,6 +1312,29 @@ curl_form_long_value(long value)
 }
 
 
+CURLcode
+curl_easy_setopt_RPGnum_(CURL *easy, CURLoption tag, curl_off_t arg)
+{
+  /* ILE/RPG procedure overloading cannot discriminate between different
+     size and/or signedness of format arguments. This provides a generic
+     wrapper that adapts size to the given tag expectation.
+     This procedure is not intended to be explicitly called from user code. */
+  if(tag / 10000 != CURLOPTTYPE_OFF_T)
+    return curl_easy_setopt(easy, tag, (long) arg);
+  return curl_easy_setopt(easy, tag, arg);
+}
+
+
+CURLcode
+curl_multi_setopt_RPGnum_(CURLM *multi, CURLMoption tag, curl_off_t arg)
+{
+  /* Likewise, for multi handle. */
+  if(tag / 10000 != CURLOPTTYPE_OFF_T)
+    return curl_multi_setopt(multi, tag, (long) arg);
+  return curl_multi_setopt(multi, tag, arg);
+}
+
+
 char *
 curl_pushheader_bynum_cssid(struct curl_pushheaders *h,
                             size_t num, unsigned int ccsid)
@@ -1458,3 +1511,24 @@ curl_easy_option_get_name_ccsid(const struct curl_easyoption *option,
 
   return (const char *) name;
 }
+
+/* Header API CCSID support. */
+CURLHcode
+curl_easy_header_ccsid(CURL *easy, const char *name, size_t index,
+                       unsigned int origin, int request,
+                       struct curl_header **hout, unsigned int ccsid)
+{
+  CURLHcode result = CURLHE_BAD_ARGUMENT;
+
+  if(name) {
+    char *s = dynconvert(ASCII_CCSID, name, -1, ccsid);
+
+    result = CURLHE_OUT_OF_MEMORY;
+    if(s) {
+      result = curl_easy_header(easy, s, index, origin, request, hout);
+      free(s);
+    }
+  }
+
+  return result;
+}
